#'  Difference-based (auto)covariance-correlation function estimation
#' 
#'  Computes \emph{and by default plots} the (auto)covariance-correlation function
#'  estimate without pre-estimating the underlying \emph{piecewise constant signal} 
#'  of the observations. To that end, a class of second-order 
#'  \emph{difference-based estimators} is implemented according to Eqs. (1.6)-(1.7)
#'  of \cite{Munk and Tecuapetla-Gómez (2015)}. By default, this function computes
#'  a subclass of estimates with minimal bias according to Eqs. (1.11)-(1.12) of 
#'  aforementioned paper.
#' 
#'  @param    data numeric vector or a univariate object of class
#'                 \code{\link[stats]{ts}} of length at least \code{2(m + 1)}.
#'  @param       m integer scalar giving the underlying level of dependency.
#'  @param       d numeric vector giving the weights used in difference-based
#'                 estimation method. If missing, the weights \code{d} are 
#'                 calculated according to Eqs. (1.11)-(1.12) of 
#'                 \cite{Munk and Tecuapetla-Gómez (2015)}.
#'                 When a single value \eqn{d^\ast}{d*} is specified, 
#'                 \code{d = rep(}\eqn{d^\ast}{d*}\code{, m + 1)}.                 
#'  @param    type character string specifying the type of acf to be computed.
#'  @param    plot logical. If \code{TRUE} (default) the acf is plotted.
#'  @param   \dots further arguments passed to \code{\link{plot.dbacf}}.
#'
#'  @note Although the theoretical properties of the methods implemented 
#'  in this function were derived for change-point regression with stationary 
#'  \emph{Gaussian} \eqn{m}-dependent errors, these methods have proven robust against 
#'  non-normality of the errors and as efficient as other methods in which 
#'  pre-estimation of an underlying smooth signal is required; the parameter
#'  \eqn{m} is essential though, see \code{\link[dbacf]{selectM}}. For further 
#'  details see Section 5 and 6 of \cite{Munk and Tecuapetla-Gómez (2015)}.
#'  
#'  @export                             
#'  
#'  @examples
#' ma2 <- arima.sim(n = 50, model = list(ma = c(0.4, -0.4), order = c(0, 0, 2)), 
#'                  sd = 0.25)
#' dbacf(ma2, m = 2)
#' 
#'  @seealso \code{\link[stats]{acf}}, \code{\link[dbacf]{plot.dbacf}}, 
#'           \code{\link[dbacf]{selectM}}
# 
#'  @return An object of class "dbacf" containing:
#'  \item{acf}{numeric vector of length \code{m + 1} giving estimated
#'             (auto)covariance-correlation.}
#'  \item{m}{integer giving underlying level of dependency.}
#'  \item{d}{numeric vector containing the weights used to estimate acf.}
#'  \item{acfType}{string indicating whether \code{covariance} or 
#'                 \code{correlation} has been computed.}
#'  \item{n}{integer giving \code{length(data)}.}
#'  \item{series}{string with name of variable \code{data}.}
#' 
#'  @references Munk, A. and Tecuapetla-Gómez, I. (2015). \emph{Autocovariance
#'  estimation in change-point regression with \eqn{m}-dependent errors: A 
#'  difference-based approach}. Submitted.
#'
dbacf <- function(data, m, d, type = c("covariance", "correlation"), plot = TRUE, 
                  ...) {
  if (!is.numeric(data)) 
    stop("'data' must be numeric")
  
  if (length(data) > 2^31 - 1) {
    stop("length of 'data' must be smaller than 2^31 - 1")
  }  
  sampleT <- length(data)
  
  series <- deparse(substitute(data))
  
  if (missing(m)) {
    stop("'m' must be specified")
  } else {
    if ( !is.wholenumber(m) || m < 0)
      stop("m must be a positive integer")
  }

  m <- as.integer(m + 2 * .Machine$double.eps ^ 0.5)        
    
  if (length(data) <= 2L * (m + 1L))
    stop("length of data must be greater than 2 * (m + 1)")
  
  if (missing(d)) {
    d <- numeric(m + 1L)
    d[1] <- 1
    if (m > 0) {
      for (i in 1:m) {
        aux <- i^2 - 4 * (m + 1 - i)^2
        if (aux >= 0) {
          d[i + 1L] <- (i + sqrt(aux)) / (2 * (m + 1 - i))
        } else {
          d[i + 1L] <- 1
        }
      }
    }      
  } else {
    if (length(d) != 1L && length(d) != m + 1L) {
      stop("length of d must be 1 or m + 1")
    }
    
    if ( !all(is.finite(d))) {
      stop("d must be a finite numeric")
    }
    
    if (length(d) == 1L) {
      d <- rep(d, m + 1L)
    }
  }
    
  type <- match.arg(type)
  
  acf <- numeric(m)
  if (m > 0) {
    for (h in 1:m)
      acf[h] <- gh(m, h, d[h + 1L], data)
  }
  acf <- c(gammaZeroEst(m, d[1], data), acf)    
  
  if (type == "correlation")
    acf <- acf / acf[1]
  
  dbacf.out <- structure(list(acf = acf, m = m,
                              d = d, acfType = type, n = sampleT, series = series), 
                         class = "dbacf")  
  
  if (plot) {
    plot.dbacf(dbacf.out, ...)
    invisible(dbacf.out)
  }
  else dbacf.out
}
#
#'  Plot of (auto)covariance-correlation estimates given by \code{\link[dbacf]{dbacf}}
#'  
#'  Plots estimated (auto)covariance-correlation given by \code{\link[dbacf]{dbacf}}.
#'  
#' 
#'  @param           x an object of class "dbacf".
#'  @param        type what type of plot should be drawn. For possible types see
#'                     \code{\link[graphics]{plot}}.
#'  @param        xlab the x label of the plot.
#'  @param        ylab the y label of the plot.
#'  @param        xlim numeric vector of length 2 giving the \code{x} coordinates
#'                     range.
#'  @param        main an overall title for the plot.
#'  @param       \dots extra arguments to be passed to plot.
#'  @param ltyZeroLine type of line used to draw horizontal line passing at 0.
#'  @param colZeroLine string indicating color of horizontal line passing at 0.
#'  
#'  @rdname plot.dbacf
#'  @method plot dbacf
#'  @export
#' 
#' @seealso \code{\link[stats]{acf}}, \code{\link[dbacf]{dbacf}}. 
#' 
plot.dbacf <- function(x, type = "h", xlab = "Lag", 
                       ylab = paste("ACF", ifelse(x$acfType == "covariance",
                                                  "(cov)", " ")),                        
                       xlim = c(0, x$m + 1), main = paste("Series", x$series),
                       ..., ltyZeroLine = 3, colZeroLine = "blue") {    
  plot(0:x$m, x$acf, type = type, xlab = xlab, 
       ylab = ylab, xlim = xlim, main = main, ...)
  abline(h = 0, lty = ltyZeroLine, col = colZeroLine)
}   
